from enum import Enum


class ClassLabLimit(Enum):
    """
    Lab limit settings.
    """
    ENFORCE = 'E'
    "Enforce class lab limit."
    INSTRUCTOR = 'I'
    "Instructor override."


class ClassEmailLogs(Enum):
    """
    Email reservation logs for real equipment pods.
    """
    NO = 'N'
    "Don't email logs."
    INSTRUCTOR = 'P'
    "Email to primary instructor."


class ClassExtensionSlots(Enum):
    """
    The number of times a reservation can be extended.
    """
    COMMUNITY = -1
    "Use the community setting."



import functools
import inspect
from distutils.version import LooseVersion

from .enums import HDRSeverity
from .errors.common import InvalidVersion


def compare_version(v1, v2):
    s1 = LooseVersion(v1)
    s2 = LooseVersion(v2)
    if s1 == s2:
        return 0
    elif s1 > s2:
        return -1
    else:
        return 1


def version_lt(v1, v2):
    return compare_version(v1, v2) > 0


def version_gte(v1, v2):
    return not version_lt(v1, v2)


def minimum_version(version):
    def decorator(f):
        @functools.wraps(f)
        def wrapper(self, *args, **kwargs):
            if version_lt(self._server_version, version):
                raise InvalidVersion(
                    '{0} is only available on NETLAB+ systems version {1} or later.'.format(
                        f.__name__, version
                    )
                )
            return f(self, *args, **kwargs)
        return wrapper
    return decorator


def hdr_result(results, severity_level=HDRSeverity.WARN):
    # TODO when card #1109 resolves, include hdr group id in response
    response = {'status': results.get('status_t'), 'value': results.get('value'), 'events': []}
    for event in results.get('events'):
        if event['severity'] <= severity_level.value:
            response['events'].append(event)
    return response


class ParamsMixin:
    def get_params(self):
        frame = inspect.currentframe()
        outer_frames = inspect.getouterframes(frame)
        caller_frame = outer_frames[1][0]

        argvalues = inspect.getargvalues(caller_frame)
        args = argvalues.args
        args.remove('self')
        values = argvalues.locals
        kwargs = argvalues.locals.get('kwargs', dict())

        params = {}
        for arg in args:
            value = values[arg]
            if value is not None:
                params[arg] = value
        params.update(kwargs)
        return params



from .common import NetlabError


CODES = []


def _add(cls):
    CODES.append(cls)
    return cls


@_add
class ReservationNotFoundError(NetlabError):
    """res_id is not found."""
    str_error = "E_RESERVATION_NOT_FOUND"


@_add
class ReservationCompletedError(NetlabError):
    """Reservation is already completed."""
    str_error = "E_RESERVATION_COMPLETED"


@_add
class ReservationNotActiveError(NetlabError):
    """res_id is not currently active."""
    str_error = "E_RESERVATION_NOT_ACTIVE"


@_add
class DateEndNotSlotError(NetlabError):
    """The specified end_time must be on a time slot boundary."""
    str_error = "E_DATE_END_NOT_SLOT"


@_add
class DateEndBeforeStartError(NetlabError):
    """The specified end_time is before start_time."""
    str_error = "E_DATE_END_BEFORE_START"


@_add
class TimeEndPassedError(NetlabError):
    """The specified end_time has already passed."""
    str_error = "E_TIME_END_PASSED"


@_add
class ReservationTimeTooShortError(NetlabError):
    """The length of the reservation is too short."""
    str_error = "E_RESERVATION_TIME_TOO_SHORT"


@_add
class DateOutOfRangeError(NetlabError):
    """Reservation is outside of dates defined for the cls_id."""
    str_error = "E_DATE_OUT_OF_RANGE"


@_add
class ClassNotMemberError(NetlabError):
    """acc_id is not part of cls_id."""
    str_error = "E_CLASS_NOT_MEMBER"


@_add
class ReservationTimeTooLongError(NetlabError):
    """Reservation is longer than the max time allowed."""
    str_error = "E_RESERVATION_TIME_TOO_LONG"


@_add
class ClassNoTeamMembersError(NetlabError):
    """There are no members in team."""
    str_error = "E_CLASS_NO_TEAM_MEMBERS"


@_add
class AccessTypeError(NetlabError):
    """Account acc_id is not an instructor."""
    str_error = "E_ACCESS_TYPE"


@_add
class PodNotAvailableError(NetlabError):
    """Pod is filtered out by pod assignment."""
    str_error = "E_POD_NOT_AVAILABLE"


@_add
class ReservationTimeMTBRError(NetlabError):
    """The minimum time between reservations has not been exceeded."""
    str_error = "E_RESERVATION_TIME_MTBR"


@_add
class ReservationConflictPodError(NetlabError):
    """This reservation conflicts with another reservation."""
    str_error = "E_RESERVATION_CONFLICT_POD"


@_add
class ReservationActivePodMaxError(NetlabError):
    """This reservation exceeds the max active pod limit."""
    str_error = "E_RESERVATION_ACTIVE_POD_MAX"


@_add
class ReservationPodRulesError(NetlabError):
    """Pod rules do not alow this reservation."""
    str_error = "E_RESERVATION_POD_RULES"


@_add
class ReservationActivePCMaxError(NetlabError):
    """This reservation exceeds the max active pc limit."""
    str_error = "E_RESERVATION_ACTIVE_PC_MAX"


@_add
class ReservationPRAMaxVirtualMachineError(NetlabError):
    """This reservation exceeds the max number of VMs for this host set by PRA."""
    str_error = "E_RESERVATION_PRA_MAX_VM"


@_add
class ReservationPRAMaxCPUError(NetlabError):
    """This reservation exceeds the max number of CPUs for this host set by PRA."""
    str_error = "E_RESERVATION_PRA_MAX_CPU"


@_add
class ReservationPRAMaxMemoryError(NetlabError):
    """This reservation exceeds the max amount of RAM for this host set by PRA."""
    str_error = "E_RESERVATION_PRA_MAX_MEM"


@_add
class ReservationTimeInsufficientError(NetlabError):
    """Not enough time remaining to complete operation."""
    str_error = "E_RESERVATION_TIME_INSUFFICIENT"



from enum import Enum


class PowerState(Enum):
    "Remote PC power state change."
    POWERED_ON = "POWERED_ON"
    "Remote PC has powered on."
    POWERED_OFF = "POWER_OFF"
    "Remote PC has powered off."
    SUSPEND_STARTED = "SUSPEND_STARTED"
    "Remote PC suspend started (vmware suspend)"
    SUSPENDED = "SUSPENDED"
    "Remote PC has suspended (vmware suspended)"
    RESUME_STARTED = "RESUME_STARTED"
    "Remote PC resume started (from vmware suspended state)"
    RESET_STARTED = "RESET_STARTED"
    "Remote PC reset started (hard reset)"
    GUEST_SHUTDOWN = "GUEST_SHUTDOWN"
    "Guest O/S shutdown started (graceful shutdown)"
    GUEST_REBOOT = "GUEST_REBOOT"
    "Guest O/S reboot started (graceful reboot)"
    GUEST_CRASHED = "GUEST_CRASHED"
    "Guest O/S has crashed"
    UNKNOWN = "UNKNOWN"
    "PC state can not be determined"


class CloneStorageAllocation(Enum):
    "How to allocate VM image."
    ONDEMAND = 'ONDEMAND'
    "Allocate on demand."
    PREALLOCATED = 'PREALLOCATED'
    "Preallocate."


class CloneType(Enum):
    "How to clone."
    LINKED = 'LINKED'
    "Delta of a full virtual machine."
    FULL = 'FULL'
    "Complete copy of the virtual machine."


class VirtualMachineLicenseType(Enum):
    "Virtual machine type."
    VHOST = "vhost"
    "License is for a vCenter server."
    VCENTER = "vcenter"
    "License is for a vm host server."



# flake8: noqa
from .klass import ClassApiMixin
from .pod import PodApiMixin
from .reservation import ReservationApiMixin
from .system import SystemApiMixin
from .user import UserApiMixin
from .vm import VmApiMixin



import click
import platform

from .api import api
from .config import config
from .utils import client
from ..errors.common import AuthenticationError, InvalidConfig
from ..version import version as client_version

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
def main():
    pass


@main.command()
@click.option('--system', default='default', help="The NETLAB+ system to get server version from. Leave blank for "
                                                  "'default'.")
def version(system):
    """ Show the client, server, Python, and OS versions"""
    try:
        results = client(system, catch_exceptions=False).system_status_get()
    except (AuthenticationError, InvalidConfig, OSError, ValueError):
        results = {}
    server_version = results.get('sys_sdn_version', click.style("Unable to connect to server", fg='red'))

    click.echo('NETLAB+ Server : {}'.format(server_version))
    click.echo('Python Client  : {}'.format(client_version))
    click.echo('Python Version : {}'.format(platform.python_version()))
    click.echo('OS Platform    : {}'.format(platform.platform()))


# manually setup top level command groups
main.add_command(api)
main.add_command(config)



from codecs import open
from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))
version = None
exec(open('netlab/version.py').read())

setup(
    name='netlab',
    version=version,

    description="NETLAB+ VE SDK for Python",
    url='https://www.netdevgroup.com/products/',

    author='Network Development Group',
    author_email='support@netdevgroup.com',

    license='MIT',

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Education',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
    ],

    # What does your project relate to?
    keywords='netlab development netlab ndg',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=['netlab', 'netlab.api', 'netlab.cli', 'netlab.enums', 'netlab.errors'],

    install_requires=[
        'click >= 6.6',
        'tabulate >= 0.7'
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev]
    extras_require={
        'dev': [
            'pytest >= 3.0.1', 'tox >= 2.4', 'Sphinx >= 1.5.0', 'recommonmark >= 0.4.0',
            'sphinx-autobuild >= 0.6.0', 'sphinx-rtd-theme == 0.1.9'
        ]
    },

    entry_points={
        'console_scripts': [
            'netlab=netlab.cli:main',
        ],
    },
)



import json
import sys
from collections import defaultdict

from paepy.ChannelDefinition import CustomSensorResult

from netlab.client import Client

if __name__ == "__main__":
    # interpret first command line parameter as json object
    if len(sys.argv) == 1:
        # for command line debugging, use the default system
        sys_args = '{"params": "default"}'
    else:
        sys_args = sys.argv[1]
    prtg = json.loads(sys_args)
    system = prtg.get('params', "default").lower()

    # PRTG runs sensors as a different user. Specify the Administrator user's netlab config file here
    client = Client(system, config_path='C:/Users/Administrator/.netlab/config.json')

    # query data from netlab api
    total = defaultdict(int)
    used = defaultdict(int)

    for pod in client.pod_list():
        total[pod['pt_id']] += 1
        if pod['pod_res_id']:
            used[pod['pt_id']] += 1

    pod_types = {pt['pt_id']: pt['pt_name'] for pt in client.pod_types_list()}

    # create sensor result
    result = CustomSensorResult("OK")

    # add primary channel for total pods reserved
    total_pods = sum(total.values())
    used_pods = sum(used.values())
    total_used_pct = (used_pods / total_pods) * 100
    result.add_channel(channel_name="Overall Percent Used", primary_channel=True, unit='Percent',
                       value=int(total_used_pct), is_float=False)

    # additional channel for each individual pod type percent, used and total
    for pt_id, total_count in total.items():
        used_count = used[pt_id]
        used_pct = (used_count / total_count) * 100
        result.add_channel(channel_name="% - {}".format(pod_types[pt_id]), unit="Percent",
                           value=int(used_pct), is_float=False)
        result.add_channel(channel_name="Total - {}".format(pod_types[pt_id]), unit="Count",
                           value=total_count, is_float=False)
        result.add_channel(channel_name="Used - {}".format(pod_types[pt_id]), unit="Count",
                           value=used_count, is_float=False)

    # print sensor result to stdout
    print(result.get_json_result())
